# API Documentation: MQ Activiti Extension: `mqPublishDelegate`

*Author*: [brian@inteligr8.com](mailto:brian@inteligr8.com)
*Since Version*: 1.0


This delegate sends a message to an MQ queue.

The process instance will continue after a successful publish of the message by this delegate to the specified queue.  Use `mqSubscribeReplyDelegate` in a subsequent task to handle the response asynchronously.

If you have more than one MQ communication in a single process, set the `mq_messageName` field on this and the corresponding `mqSubscribeReplyDelegate` task.  The resultant variables will then include that message name (e.g. `mq_messageId_{mq_messageName}`).

*See Also*: [`mqSubscribeReplyDelegate`](bean-mqSubscribeReplyDelegate.md)
*See Also*: [`mqSubscribeDelegate`](bean-mqSubscribeDelegate.md)

## <a name="delegate"></a> Delegate Expression Uses

This bean may be used as a *Delegate* using the "Delegate Expression" field as shown in the snippet below.

```javascript
${mqPublishDelegate}
```

You may use it in a **Service Task**.


This method makes this bean an Activiti delegate.

| Input Type               | Name                     | Java Type                                        | Documentation                      |
| ------------------------ | ------------------------ | ------------------------------------------------ | ---------------------------------- |
| BPMN Field               | `mq_connectorId`         |                                                  | An Activiti App Tenant Endpoint ID or Java system property in the format `inteligr8.mq.connectors.{connectorId}.url`.  Using system properties support `url`, `username`, and `password`. |
| BPMN Field               | `mq_queueName`           |                                                  | The name of an MQ destination (queue or topic).  This is the target of the message.  If it doesn't exist, it will be created. |
| BPMN Field               | `mq_metadataProcessScope` |                                                  | [optional] `true` to set the `mq_messageId` result variable in the process instance scope; otherwise the variable will be local to the task.  `mq_correlationId` is always at the process scope. |
| BPMN Field               | `mq_messageName`         |                                                  | [optional] A unique identifier to append to Activiti result variable names when more than one MQ message publication/subscription is supported by a process. |
| BPMN Field               | `mq_priority`            |                                                  | [optional] A priority of the MQ message.  May be an expression.  Value depends on MQ protocol. |
| BPMN Field               | `mq_payload`             |                                                  | [optional] The body of the MQ message.  May include expressions. |
| BPMN Field               | `mq_payloadMimeType`     |                                                  | [optional] The MIME type of the body of the MQ message. |
| BPMN Field               | `mq_replyQueueName`      |                                                  | [optional] The name of an MQ destination (queue or topic).  This tells the processor of the message where to send a reply. |
| BPMN Field               | `mq_statusQueueName`     |                                                  | [optional] The name of an MQ destination (queue or topic).  This tells the processor of the message where to send status updates. |
| Activiti Variable        | `mq_correlationId`       |                                                  | [optional] The correlationId of the message to send. |

| Result Type              | Java Type, Name, or Error Code                   | Documentation                    |
| ------------------------ | ------------------------------------------------ | -------------------------------- |
| Activiti Variable        | `mq_correlationId`                               | The correlationId of the message sent. |
| Activiti Variable        | `mq_messageId`                                   | The messageId of the message sent. |
| Thrown BPMN Error        | `timeout`                                        | The MQ connection timed out connecting or waiting for a message. |
| Thrown BPMN Error        | `network`                                        | The MQ connection experienced network issues. |
| Thrown BPMN Error        | `mq`                                             | An unknown MQ issue occurred.    |

*See Also*: [`mqSubscribeReplyDelegate`](bean-mqSubscribeReplyDelegate.md#delegate)
*See Also*: [`mqSubscribeDelegate`](bean-mqSubscribeDelegate.md#delegate)

---

[Back to Index](index.md)

Generated by the [Inteligr8](https://inteligr8.com) [Activiti API Doclet](https://bitbucket.org/inteligr8/activiti-api-doclet).

