package com.inteligr8.activiti.mq;

import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import javax.jms.JMSException;

import org.activiti.engine.delegate.JavaDelegate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;

import com.activiti.domain.idm.EndpointConfiguration;
import com.activiti.service.api.EndpointService;

public abstract class AbstractMqDelegate implements JavaDelegate {
    
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private final String propertyPrefix = "inteligr8.mq";

    @Autowired
    private Environment env;

    @Autowired
    private EndpointService endpointService;
    
    @Autowired
    private MqCommunicationService mqCommService;

	private Map<String, MqCommunicator> communicators = new HashMap<>();
    
    protected synchronized MqCommunicator getCommunicator(String connectorId) throws JMSException, TimeoutException, IOException {
        MqCommunicator communicator = this.communicators.get(connectorId);
        if (communicator == null) {
            EndpointConfiguration endpointConfig = this.endpointService.getConfigurationByName(connectorId);
            if (endpointConfig != null) {
                communicator = this.mqCommService.getCommunicator(endpointConfig);
            } else {
                this.logger.debug("Cannot find APS endpoint declaration; looking in properties for configuration: {}", connectorId);
                String propertyKeyPrefix = this.propertyPrefix + ".connectors." + connectorId;
                String urlPropertyKey = propertyKeyPrefix + ".url";
                String url = this.env.getProperty(urlPropertyKey);
                if (url == null)
                    throw new IllegalStateException("The '" + connectorId + "' APS endpoint (" + connectorId + ") or URL property (" + urlPropertyKey + ") could not be found.");

                String username = this.env.getProperty(propertyKeyPrefix + ".username");
                String password = this.env.getProperty(propertyKeyPrefix + ".password");
                communicator = this.mqCommService.getCommunicator(URI.create(url), username, password);
            }

            this.communicators.put(connectorId, communicator);
        }

        return communicator;
    }
    
    protected Object mapValueToVariableValue(Object jmsValue) {
    	// TODO
    	return jmsValue;
    }

}
