package com.inteligr8.activiti.mq;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;

public class ByteArrayChannel implements ReadableByteChannel {
	
	private final ByteBuffer bytes;
	private boolean closed = false;
	
	public ByteArrayChannel(byte[] bytes) {
		this.bytes = ByteBuffer.wrap(bytes);
	}
	
	@Override
	public void close() throws IOException {
		this.closed = true;
	}
	
	@Override
	public boolean isOpen() {
		return !this.closed;
	}
	
	@Override
	public int read(ByteBuffer dst) throws IOException {
		if (!this.bytes.hasRemaining())
			return -1;
		if (!dst.hasRemaining())
			return 0;
		
		int bytesToWrite = Math.min(this.bytes.remaining(), dst.remaining());
		this.bytes.limit(this.bytes.position() + bytesToWrite);
		dst.put(this.bytes);
			
		return bytesToWrite;
	}

}
