package com.inteligr8.activiti.mq;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import javax.jms.JMSException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import com.activiti.domain.idm.EndpointConfiguration;
import com.inteligr8.activiti.mq.amqp.AmqpCommunicator;
import com.inteligr8.activiti.mq.jms.JmsCommunicator;

@Component
public class MqCommunicationService {
	
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
	private Map<String, MqCommunicator> communicators = new HashMap<>();
	
	public MqCommunicator getCommunicator(EndpointConfiguration endpoint) throws JMSException {
		return this.getCommunicator(
			endpoint.getProtocol().getProtocol(),
			endpoint.getUrl(),
			endpoint.getBasicAuth().getUsername(),
			endpoint.getBasicAuth().getPassword());
	}
	
	public MqCommunicator getCommunicator(URI url, String username, String password) throws JMSException {
		return this.getCommunicator(
			url.getScheme(),
			url.toString(),
			username,
			password);
	}
	
	protected synchronized MqCommunicator getCommunicator(String protocol, String url, String username, String password) throws JMSException {
		String key = url + "@" + username;
		MqCommunicator communicator = this.communicators.get(key);
		if (communicator == null) {
			switch (protocol.toLowerCase()) {
				case "ampq":
					this.logger.debug("Creating AMQP connector: {}", url);
					communicator = new AmqpCommunicator(url, username, password);
					break;
				case "failover":
				case "tcp":
				case "http":
					this.logger.debug("Creating TCP OpenWire connector: {}", url);
					communicator = new JmsCommunicator(url, username, password);
					break;
				case "ssl":
				case "https":
					this.logger.debug("Creating SSL OpenWire connector: {}", url);
					communicator = new JmsCommunicator(url, username, password);
					break;
				default:
					throw new IllegalArgumentException("The " + protocol + " is not supported by this library");
			}
			
			this.communicators.put(key, communicator);
		}
		
		return communicator;
	}

}
