package com.inteligr8.activiti.mq;

import java.time.OffsetDateTime;
import java.util.Date;

import org.activiti.engine.ProcessEngine;
import org.activiti.engine.delegate.DelegateExecution;

public class MqDelegateExecution {
	
	protected final DelegateExecution execution;
	protected final MqServiceTask task;
	
	public MqDelegateExecution(ProcessEngine services, MqServiceTaskService msts, DelegateExecution execution) {
		this.execution = execution;
		this.task = msts.get(execution.getProcessDefinitionId(), execution.getCurrentFlowElement());
	}
	
	public void validate() {
	}

	/**
	 * Unlike MqServiceTask, this allows for expression expansion based on the variables
	 */
	protected <T> T getFieldValueFromModel(String fieldName, Class<T> type) {
		return this.task.getFieldValueFromModel(fieldName, type, this.execution, false);
	}
	
	/**
	 * Unlike MqServiceTask, this allows for expression expansion based on the variables; even partial values
	 */
	protected <T> T getFieldValueFromModel(String fieldName, Class<T> type, boolean forceExpressionProcessing) {
		return this.task.getFieldValueFromModel(fieldName, type, this.execution, forceExpressionProcessing);
	}
	
	public String getConnectorIdFromModel() {
		return this.getFieldValueFromModel(Constants.FIELD_CONNECTOR_ID, String.class);
	}
	
	public String getQueueNameFromModel() {
		return this.getFieldValueFromModel(Constants.FIELD_QUEUE_NAME, String.class);
	}
	
	public String getMessageNameFromModel() {
		return this.getFieldValueFromModel(Constants.FIELD_MESSAGE_NAME, String.class);
	}
	
	public String getPayloadFromModel() {
		return this.getFieldValueFromModel(Constants.FIELD_PAYLOAD, String.class, true);
	}
	
	public String getPayloadMimeTypeFromModel() {
		return this.getFieldValueFromModel(Constants.FIELD_PAYLOAD_MIME_TYPE, String.class);
	}
    
    protected void setMqVariable(String varName, Object value) {
    	varName = this.formulateVariableName(varName);
    	if (this.task.doWriteToProcessScope()) {
    		this.execution.setVariable(varName, value);
    	} else {
    		this.execution.setVariableLocal(varName, value);
    	}
    }
	
	public String formulateVariableName(String varName) {
		if (this.getMessageNameFromModel() != null)
			varName += "_" + this.getMessageNameFromModel();
		return varName;
	}

	/**
	 * Always pull from the process scope.
	 * @param correlationId
	 */
	public String getCorrelationId() {
		return this.execution.getVariable(this.formulateVariableName(Constants.VARIABLE_CORRELATION_ID), String.class);
	}
	
	/**
	 * Always set at process scope.
	 * @param correlationId
	 */
	public void setCorrelationId(String correlationId) {
		this.execution.setVariable(this.formulateVariableName(Constants.VARIABLE_CORRELATION_ID), correlationId);
	}
	
	public void setMessageId(String messageId) {
		this.setMqVariable(Constants.VARIABLE_MESSAGE_ID, messageId);
	}
	
	public void setDeliveryTime(OffsetDateTime deliveryTime) {
		if (deliveryTime != null)
			this.setMqVariable(Constants.VARIABLE_DELIVERY_TIME, Date.from(deliveryTime.toInstant()));
	}
	
	public void setPriority(Integer priority) {
		if (priority != null)
			this.setMqVariable(Constants.VARIABLE_PRIORITY, priority);
	}
	
	public void setReplyToQueueName(String replyToQueueName) {
		if (replyToQueueName != null)
			this.setMqVariable(Constants.VARIABLE_REPLY_QUEUE_NAME, replyToQueueName);
	}
	
	public void setStatusQueueName(String statusQueueName) {
		if (statusQueueName != null)
			this.setMqVariable(Constants.VARIABLE_STATUS_QUEUE_NAME, statusQueueName);
	}
	
	public void setPayload(String payload, String mimeType) {
		this.setPayload(payload);
		this.setPayloadMimeType(mimeType);
	}
	
	public void setPayload(String payload) {
		if (payload != null)
			this.setMqVariable(Constants.VARIABLE_PAYLOAD, payload);
	}
	
	public void setPayloadMimeType(String payloadMimeType) {
		if (payloadMimeType != null)
			this.setMqVariable(Constants.VARIABLE_PAYLOAD_MIME_TYPE, payloadMimeType);
	}

}
