package com.inteligr8.activiti.mq;

import org.activiti.engine.delegate.event.ActivitiEventType;
import org.activiti.engine.impl.persistence.entity.Entity;
import org.activiti.engine.impl.persistence.entity.JobEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * This needs to be changed to detect just process versioning (deactivation?)
 */
@Component
public class MqJobEventListener implements ActivitiEntityEventListener<JobEntity> {

	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	protected MqSubscriptionService subscriptionService;
	
	@Override
	public boolean ofEntityType(Class<? extends Entity> entityType) {
		return JobEntity.class.isAssignableFrom(entityType);
	}
	
	@Override
	public boolean ofEntityType(Entity entity) {
		return entity instanceof JobEntity;
	}
	
	@Override
	public void onEntityEvent(ActivitiEventType eventType, String processDefinitionId, String processInstanceId,
			String executionId, JobEntity entity) {
		this.logger.trace("Triggered by job event: {}", entity);
		
		// switch (eventType) {
		// 	case ENTITY_DELETED:
		// 	case ENTITY_SUSPENDED:
		// 		this.onJobRemoveEvent(entity);
		// 		break;
		// 	default:
		// }
	}
	
	protected void onJobRemoveEvent(JobEntity entity) {
		this.logger.trace("Triggered by job removal: {}", entity);
		try {
			this.subscriptionService.cancel(entity.getExecutionId());
			this.logger.debug("Subscription execution ended due to job removal: job: {}: exec: {}", entity.getId(), entity.getExecutionId());
		} catch (Exception e) {
			this.logger.error("Subscription execution failed to be canceled after job removal: job: " + entity.getId() + ": exec: " + entity.getExecutionId(), e);
		}
	}

}
