package com.inteligr8.activiti.mq;

import java.util.HashMap;
import java.util.Map;

import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowElement;
import org.activiti.bpmn.model.ServiceTask;
import org.activiti.engine.ProcessEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class MqServiceTaskService {
	
	private final Logger logger = LoggerFactory.getLogger(MqServiceTaskService.class);
	
	@Autowired
	private ProcessEngine services;
	
	/**
	 * This is an indefinite cache with no updates or removals.
	 * We should be fine here.
	 */
	private Map<String, MqServiceTask> taskCache = new HashMap<>();
	
	public MqServiceTask get(String processDefinitionId, String taskId) {
		String key = processDefinitionId + "|" + taskId;
		if (this.taskCache.containsKey(key))
			return this.taskCache.get(key);
		
		BpmnModel model = this.services.getRepositoryService().getBpmnModel(processDefinitionId);
		if (model == null)
			throw new IllegalArgumentException("The process definition '" + processDefinitionId + "' could not be found");
		
		FlowElement element = model.getFlowElement(taskId);
		if (element == null)
			throw new IllegalArgumentException("The service task '" + taskId + "' could not be found in the process definition '" + processDefinitionId + "' model");
		
		if (!(element instanceof ServiceTask))
			throw new IllegalArgumentException("The model element '" + taskId + "' is not a service task as expected");
		
		ServiceTask task = (ServiceTask) element;
		this.logger.trace("Discovered service task: {}: {}", task.getId(), task.getName());
		
		MqServiceTask mqtask = new MqServiceTask(this.services, task);
		mqtask.validate();
		this.taskCache.put(key, mqtask);
		return mqtask;
	}
	
	public MqServiceTask get(String processDefinitionId, FlowElement taskElement) {
		String key = processDefinitionId + "|" + taskElement.getId();
		if (this.taskCache.containsKey(key))
			return this.taskCache.get(key);
		
		if (!(taskElement instanceof ServiceTask))
			throw new IllegalArgumentException("The model element '" + taskElement.getId() + "' is not a service task as expected");
		
		ServiceTask task = (ServiceTask) taskElement;
		this.logger.trace("Discovered service task: {}: {}", task.getId(), task.getName());
		
		MqServiceTask mqtask = new MqServiceTask(this.services, task);
		mqtask.validate();
		this.taskCache.put(key, mqtask);
		return mqtask;
	}
	
	public MqServiceTask get(String processDefinitionId, ServiceTask task) {
		String key = processDefinitionId + "|" + task.getId();
		if (this.taskCache.containsKey(key))
			return this.taskCache.get(key);
		
		MqServiceTask mqtask = new MqServiceTask(this.services, task);
		mqtask.validate();
		this.taskCache.put(key, mqtask);
		return mqtask;
	}

}
