package com.inteligr8.activiti.mq.amqp;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.ReadableByteChannel;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.inteligr8.activiti.mq.ByteArrayChannel;
import com.inteligr8.activiti.mq.model.AbstractMessage;
import com.inteligr8.activiti.mq.model.DeliveredMessage;
import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Delivery;

/**
 * FIXME this is completley untested
 */
public class AmqpDeliveredMessage<BodyType> extends AbstractMessage implements DeliveredMessage<BodyType> {
	
	@SuppressWarnings("unchecked")
	public AmqpDeliveredMessage(Delivery message, Class<BodyType> bodyType) throws IOException {
		this.setCorrelationId(message.getProperties().getCorrelationId());
		this.setMessageId(message.getProperties().getMessageId());
		this.setPriority(message.getProperties().getPriority());
		if (message.getProperties().getTimestamp() != null)
			this.setDeliveryTime(OffsetDateTime.ofInstant(message.getProperties().getTimestamp().toInstant(), ZoneOffset.systemDefault()));
		this.setReplyToQueueName(message.getProperties().getReplyTo());
		this.setContentType(message.getProperties().getContentType());
		
		String encoding = message.getProperties().getContentEncoding() == null ? "utf-8" : message.getProperties().getContentEncoding();
		if (String.class.isAssignableFrom(bodyType.getClass())) {
			this.setContent((BodyType) new String(message.getBody(), encoding));
		} else if (byte[].class.isAssignableFrom(bodyType)) {
			this.setContent((BodyType) message.getBody());
		} else if (InputStream.class.isAssignableFrom(bodyType)) {
			this.setContent((BodyType) new ByteArrayInputStream(message.getBody()));
		} else if (ReadableByteChannel.class.isAssignableFrom(bodyType)) {
			this.setContent((BodyType) new ByteArrayChannel(message.getBody()));
		} else {
			String str = new String(message.getBody(), encoding);
			ObjectMapper om = new ObjectMapper();
			this.setContent(om.readValue(str, bodyType));
		}
		
		if (message.getProperties().getHeaders() != null)
			this.getProperties().putAll(message.getProperties().getHeaders());
	}
	
	public AmqpDeliveredMessage(AMQP.BasicProperties props, Class<BodyType> bodyType) throws IOException {
		this.setMessageId(props.getMessageId());
		this.setCorrelationId(props.getCorrelationId());
	}
	
	public static <BodyType> AmqpDeliveredMessage<BodyType> transform(AMQP.BasicProperties props) throws IOException {
		return new AmqpDeliveredMessage<>(props, null);
	}
	
	public static <BodyType> AmqpDeliveredMessage<BodyType> transform(AMQP.BasicProperties props, Class<BodyType> bodyType) throws IOException {
		return new AmqpDeliveredMessage<>(props, bodyType);
	}
	
	public static <BodyType> AmqpDeliveredMessage<BodyType> transform(Delivery message) throws IOException {
		return new AmqpDeliveredMessage<>(message, null);
	}
	
	public static <BodyType> AmqpDeliveredMessage<BodyType> transform(Delivery message, Class<BodyType> bodyType) throws IOException {
		return new AmqpDeliveredMessage<>(message, bodyType);
	}

}
