package com.inteligr8.activiti.mq.amqp;

import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.util.Date;

import javax.jms.DeliveryMode;

import org.apache.commons.io.IOUtils;
import org.jgroups.util.UUID;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.inteligr8.activiti.mq.model.AbstractMessage;
import com.inteligr8.activiti.mq.model.PreparedMessage;
import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Delivery;

/**
 * FIXME this is completely untested
 */
public class AmqpPreparedMessage<BodyType> extends AbstractMessage implements PreparedMessage<BodyType> {
	
	public Delivery transform(Class<BodyType> bodyType) throws IOException {
		if (this.getContentType() == null) {
			if (String.class.isAssignableFrom(bodyType)) {
				this.setContentType("text/plain");
			} else if (byte[].class.isAssignableFrom(bodyType)) {
				this.setContentType("application/octet-stream");
			} else if (InputStream.class.isAssignableFrom(bodyType)) {
				this.setContentType("application/octet-stream");
			} else {
				// if not set, we are going to force it to JSON using the JackSON mapper
				this.setContentType("application/json");
			}
		}
		
		String correlationId = this.getCorrelationId();
		if (correlationId == null)
			correlationId = UUID.randomUUID().toString();
		
		AMQP.BasicProperties bprops = new AMQP.BasicProperties(
				this.getContentType(),
				"utf-8",
				this.getProperties(),
				DeliveryMode.PERSISTENT, // yes, this is a JMS class, but the values are the same; survive restarts
				this.getPriority(),
				correlationId,
				this.getReplyToQueueName() != null ? this.getReplyToQueueName() : null,
				null, // expiration
				null, // messageId
				new Date(), // timestamp
				null, // type
				null, // userId
				null, // appId
				null  // clusterId
		);
		
		byte[] content = null;
		if (String.class.isAssignableFrom(bodyType)) {
			content = ((String) this.getContent()).getBytes("utf-8");
		} else if (byte[].class.isAssignableFrom(bodyType)) {
			content = (byte[]) this.getContent();
		} else if (InputStream.class.isAssignableFrom(bodyType)) {
			content = IOUtils.toByteArray((InputStream) this.getContent());
		} else if (ReadableByteChannel.class.isAssignableFrom(bodyType)) {
			InputStream istream = Channels.newInputStream((ReadableByteChannel) this.getContent());
			content = IOUtils.toByteArray(istream);
		} else {
			ObjectMapper om = new ObjectMapper();
			content = om.writeValueAsString(this.getContent()).getBytes("utf-8");
		}

		return new Delivery(null, bprops, content);
	}

}
