package com.inteligr8.activiti.mq.jms;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map.Entry;

import javax.jms.BytesMessage;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.StreamMessage;

import org.apache.commons.io.IOUtils;
import org.jgroups.util.UUID;

import com.inteligr8.activiti.mq.model.AbstractMessage;
import com.inteligr8.activiti.mq.model.PreparedMessage;

public class JmsPreparedMessage<BodyType> extends AbstractMessage implements PreparedMessage<BodyType> {
	
	public Message transform(Session session) throws JMSException {
		Message message = null;
		
		try {
			if (this.getContent() == null) {
				message = session.createMessage();
			} else if (this.getContent() instanceof String) {
				message = session.createTextMessage((String) this.getContent());
			} else if (this.getContent() instanceof byte[]) {
				BytesMessage bmessage = session.createBytesMessage();
				IOUtils.write((byte[]) this.getContent(), new BytesMessageOutputStream(bmessage));
				message = bmessage;
			} else if (this.getContent() instanceof InputStream) {
				StreamMessage smessage = session.createStreamMessage();
				IOUtils.write((byte[]) this.getContent(), new StreamMessageOutputStream(smessage));
				message = smessage;
			} else {
				throw new IllegalArgumentException();
			}
		} catch (IOException ie) {
			throw (JMSException) ie.getCause();
		}
		
		if (this.getCorrelationId() == null) {
			message.setJMSCorrelationID(UUID.randomUUID().toString());
		} else {
			message.setJMSCorrelationID(this.getCorrelationId());
		}
		
		if (this.getProperties() != null) {
			for (Entry<String, Object> prop : this.getProperties().entrySet())
				message.setObjectProperty(prop.getKey(), prop.getValue());
		}
		
		if (this.getReplyToQueueName() != null)
			message.setJMSReplyTo(session.createQueue(this.getReplyToQueueName()));
		
		if (this.getPriority() != null)
			message.setJMSPriority(this.getPriority());
		
		message.setJMSDeliveryMode(DeliveryMode.PERSISTENT); // survive restarts
		return message;
	}

}
