package com.inteligr8.activiti.mq.model;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.concurrent.NotThreadSafe;

@NotThreadSafe
public interface PreparedMessage<BodyType> extends Message {
	
	default String getCorrelationId() {
		return this.getProperty("correlationId");
	}
	
	default void setCorrelationId(String correlationId) {
		this.setProperty("correlationId", correlationId);
	}
	
	default Map<String, Object> getProperties() {
		return this.getProperty("properties");
	}
	
	// not thread safe
	default void setProperty(String key, Object value) {
		Map<String, Object> properties = this.getProperties();
		if (properties == null)
			properties = new HashMap<>();
		this.setProperty("properties", properties);
		
		if (value == null) {
			properties.remove(key);
		} else {
			properties.put(key, value);
		}
	}
	
	default Integer getPriority() {
		return this.getProperty("priority");
	}
	
	default void setPriority(Integer priority) {
		this.setProperty("priority", priority);
	}
	
	default String getReplyToQueueName() {
		return this.getProperty("replyToQueueName");
	}
	
	default void setReplyToQueueName(String replyToQueueName) {
		this.setProperty("replyToQueueName", replyToQueueName);
	}
	
	default String getStatusQueueName() {
		return this.getProperty("statusQueueName");
	}
	
	default void setStatusQueueName(String statusQueueName) {
		this.setProperty("statusQueueName", statusQueueName);
	}
	
	default BodyType getContent() {
		return this.getProperty("content");
	}
	
	default void setContent(BodyType content) {
		this.setProperty("content", content);
	}
	
	default String getContentType() {
		return this.getProperty("contentType");
	}
	
	default void setContentType(String contentType) {
		this.setProperty("contentType", contentType);
	}
	
	default void setContent(BodyType content, String contentType) {
		this.setContent(content);
		this.setContentType(contentType);
	}

}
